<?php

declare(strict_types=1);

namespace mirzaev\parser_from_interneturok\models\telegram;

// Files of the project
use mirzaev\parser_from_interneturok\models\core;

// Framework for Telegram
use Zanzara\Context as context,
	Zanzara\Middleware\MiddlewareNode as node;

/**
 * Telegram middlewares
 *
 * @package mirzaev\parser_from_interneturok\models\telegram
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class middlewares extends core
{
	/**
	 * Authorization (middleware)
	 *
	 * Checks account permissions against the registry
	 *
	 * @param context $context
	 * @param node $next
	 *
	 * @return void
	 */
	public static function authorization(context $context, node $next): void
	{
		// Is the process stopped?
		if ($context->get('stop')) return;

		// Initializing the telegram account
		$telegram = $context->getEffectiveUser();

		if (array_search($telegram->getId(), ACCOUNTS) !== false) {
			// Authorized the account

			// Continuation of the process
			$next($context);
		} else {
			// Not authorized the account	

			// Sending the message
			$context->sendMessage('⛔️ У тебя нет доступа к чат\-роботу')->then(function () use ($context) {
				// Sended the message

				// Ending the conversation
				$context->endConversation();
			});
		}
	}
}
