<?php

declare(strict_types=1);

namespace mirzaev\parser_from_interneturok\models\telegram;

// Files of the project
use mirzaev\parser_from_interneturok\models\core,
	mirzaev\parser_from_interneturok\models\interneturok,
	mirzaev\parser_from_interneturok\models\enumerations\subject;

// Framework for Telegram
use Zanzara\Context as context,
	Zanzara\Telegram\Type\Message as message,
	Zanzara\Telegram\Type\Input\InputFile as file_input;

// Framework for asynchronous PHP
use function React\Async\await;

// Built-in libraries
use Exception as exception,
	DateTime as datetime;

/**
 * Telegram shcool subjects parser
 *
 * @package mirzaev\parser_from_interneturok\models\telegram
 *
 * @license http://www.wtfpl.net/ Do What The Fuck You Want To Public License
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class parser extends core
{
	/**
	 * Process
	 *
	 * @var const string PROCESS Name of the process in the telegram user buffer
	 */
	public const string PROCESS = 'parse_subject';

	/**
	 * Subject
	 *
	 * Create the process, write subject to the buffer and request grade
	 *
	 * @param context $context Request data from Telegram
	 * @param subject $subject The schrool subject
	 *
	 * @return void
	 */
	public static function subject(context $context, subject $subject): void
	{
		// Initializing the parsing target buffer
		$target = [
			'subject' => $subject,
			'grade' => null,
			'date' => null,
			'week' => null
		];

		// Writing to the telegram user buffer
		$context->setUserDataItem(static::PROCESS, $target)
			->then(function () use ($context) {
				// Writed to the telegram user buffer

				// Sending the message
				$context->sendMessage(
					'🎒 *Выбери номер класса*',
					[
						'reply_markup' => [
							'inline_keyboard' => [
								[
									[
										'text' => '4',
										'callback_data' => 'grade_4'
									],
									[
										'text' => '5',
										'callback_data' => 'grade_5'
									],
									[
										'text' => '6',
										'callback_data' => 'grade_6'
									],
									[
										'text' => '7',
										'callback_data' => 'grade_7'
									]
								],
								[
									[
										'text' => '8',
										'callback_data' => 'grade_8'
									],
									[
										'text' => '9',
										'callback_data' => 'grade_9'
									],
									[
										'text' => '10',
										'callback_data' => 'grade_10'
									],
									[
										'text' => '11',
										'callback_data' => 'grade_11'
									]
								]
							],
							'disable_notification' => true,
							'remove_keyboard' => true
						]
					]
				);
			});
	}

	/**
	 * Grade
	 *
	 * Write grade to the buffer and request date
	 *
	 * @param context $context Request data from Telegram
	 * @param int $grade The grade
	 *
	 * @return void
	 */
	public static function grade(context $context, int $grade): void
	{
		// Reading from the telegram user buffer
		$context->getUserDataItem(static::PROCESS)
			->then(function (?array $target) use ($context, $grade) {
				// Readed from the telegram user buffer

				if ($target) {
					// Initialized the parsing process

					// Initializing the grade
					$target['grade'] = $grade;

					// Writing to the telegram user buffer
					$context->setUserDataItem(static::PROCESS, $target)
						->then(function () use ($context, $target) {
							// Writed to the telegram user buffer

							// Sending the message
							$context->sendMessage(
								'📅 *Выберите номер недели*',
								[
									'reply_markup' => [
										'inline_keyboard' => [
											[
												[
													'text' => '2',
													'callback_data' => 'week_2'
												],
												[
													'text' => '3',
													'callback_data' => 'week_3'
												],
												[
													'text' => '4',
													'callback_data' => 'week_4'
												],
												[
													'text' => '5',
													'callback_data' => 'week_5'
												],
												[
													'text' => '6',
													'callback_data' => 'week_6'
												],
												[
													'text' => '7',
													'callback_data' => 'week_7'
												],
												[
													'text' => '8',
													'callback_data' => 'week_8'
												],
												[
													'text' => '9',
													'callback_data' => 'week_9'
												]
											],
											[
												[
													'text' => '11',
													'callback_data' => 'week_11'
												],
												[
													'text' => '12',
													'callback_data' => 'week_12'
												],
												[
													'text' => '13',
													'callback_data' => 'week_13'
												],
												[
													'text' => '14',
													'callback_data' => 'week_14'
												],
												[
													'text' => '15',
													'callback_data' => 'week_15'
												],
												[
													'text' => '16',
													'callback_data' => 'week_16'
												],
												[
													'text' => '17',
													'callback_data' => 'week_17'
												],
												[
													'text' => '18',
													'callback_data' => 'week_18'
												]
											],
											[
												[
													'text' => '21',
													'callback_data' => 'week_21'
												],
												[
													'text' => '22',
													'callback_data' => 'week_22'
												],
												[
													'text' => '23',
													'callback_data' => 'week_23'
												],
												[
													'text' => '24',
													'callback_data' => 'week_24'
												],
												[
													'text' => '25',
													'callback_data' => 'week_25'
												]
											],
											[
												[
													'text' => '26',
													'callback_data' => 'week_26'
												],
												[
													'text' => '27',
													'callback_data' => 'week_27'
												],
												[
													'text' => '28',
													'callback_data' => 'week_28'
												],
												[
													'text' => '29',
													'callback_data' => 'week_29'
												],
												[
													'text' => '30',
													'callback_data' => 'week_30'
												]
											],
											[
												[
													'text' => '32',
													'callback_data' => 'week_32'
												],
												[
													'text' => '33',
													'callback_data' => 'week_33'
												],
												[
													'text' => '34',
													'callback_data' => 'week_34'
												],
												[
													'text' => '35',
													'callback_data' => 'week_35'
												],
												[
													'text' => '36',
													'callback_data' => 'week_36'
												],
												[
													'text' => '37',
													'callback_data' => 'week_37'
												],
												[
													'text' => '38',
													'callback_data' => 'week_38'
												]
											]
										],
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							);
						});
				} else {
					// Not initialized the parsing process

					// Sending the message
					$context->sendMessage('⚠️ *Не запущен процесс генерации парсинга*')
						->then(function (message $message) use ($context) {
							// Sended the message

							// Sending the menu with subjects
							commands::menu($context);
						});
				}
			});
	}

	/**
	 * Date
	 *
	 * Write date and request confirmation for starting the parsing process
	 *
	 * @param context $context Request data from Telegram
	 * @param int $week Number of the week
	 *
	 * @return void
	 */
	public static function date(context $context, int $week): void
	{
		// Reading from the telegram user buffer
		$context->getUserDataItem(static::PROCESS)
			->then(function (?array $target) use ($context, $week) {
				// Readed from the telegram user buffer

				if ($target) {
					// Initialized the parsing process

					// Initializing the week
					$target['week'] = $week;

					// Initializing the date
					$target['date'] = interneturok::week($week);

					// Writing to the telegram user buffer
					$context->setUserDataItem(static::PROCESS, $target)
						->then(function () use ($context, $target) {
							// Writed to the telegram user buffer

							// Initializing the subject
							$subject = $target['subject']?->value;

							// Initializing the grade
							$grade = $target['grade'];

							// Reinigitlizing the date
							$date = $target['date']->format('Y\\\.m\\\.d');

							// Inigitlizing number the week
							$week = $target['week'];

							// Sending the message
							$context->sendMessage(
								<<<TXT
										📦 *Генерация запроса*

										*Предмет:* $subject
										*Класс:* $grade
										*Дата:* $date \($week\)
										TXT,
								[
									'reply_markup' => [
										'inline_keyboard' => [
											[
												[
													'text' => '✅ Отправить запрос',
													'callback_data' => 'parse'
												]
											]
										],
										'disable_notification' => true,
										'remove_keyboard' => true
									]
								]
							);
						});
				} else {
					// Not initialized the parsing process

					// Sending the message
					$context->sendMessage('⚠️ *Не запущен процесс генерации парсинга*')
						->then(function (message $message) use ($context) {
							// Sended the message

							// Sending the menu with subjects
							commands::menu($context);
						});
				}
			});
	}

	/**
	 * Parse
	 *
	 * @param context $context Request data from Telegram
	 *
	 * @return void
	 */
	public static function parse(context $context): void
	{
		// Reading from the telegram user buffer
		$context->getUserDataItem(static::PROCESS)
			->then(function (?array $target) use ($context) {
				// Readed from the telegram user buffer

				if ($target) {
					// Initialized the parsing process

					// Sending the message
					$context->sendMessage('⚡️ *Запущен процесс парсинга домашнего задания*')
						->then(function (message $message) use ($context, $target) {
							// Sended the message

							// Initializing variables for the parsing process
							$subject = $target['subject'];
							$grade = $target['grade'];
							$date = $target['date'];
							$week = $target['week'];

							// Initializing the parser
							$parser = new interneturok();

							try {
								// Parsing homework
								$homework = $parser->parse(subject: $subject, grade: $grade, date: $date, week: $week, waiting: 2);

								if (!empty($homework)) {
									// Initialized homework

									foreach ($homework['files'] as $file) {
										// Iterating over homework files in the storage

										// Sending the file
										await($context->sendDocument(new file_input($file)));
									}

									if ($homework['downloads']['year'] <= 100) {
										// The homework was downloadsd 100 or less times for 1 last year

										// Sending the message
										$context->sendMessage('Домашнее задание было скачано *' . $homework['downloads']['year'] . ' раз* за последний год');
									} else {
										// The homework was downloaded more than 100 times for 1 last year

										// Sending the message
										$context->sendMessage('Домашнее задание было скачано *более чем 100 раз*) за последний год');
									}

									// Deleting in the telegram user buffer
									$context->deleteUserDataItem(static::PROCESS);
								} else {
									// Not initialized homework files

									// Sending the message
									$context->sendMessage('❌ *Не найдены домашние задания*')
										->then(function (message $message) use ($context) {
											// Sended the message

											// Sending the menu with subjects
											commands::menu($context);
										});
								}
							} catch (exception $exception) {
								// Sending the message
								$context->sendMessage('⚠️  *Произошла ошибка при обработке домашних заданий*')
									->then(function (message $message) use ($context) {
										// Sended the message

										// Sending the menu with subjects
										commands::menu($context);
									});
							}
						});
				} else {
					// Not initialized the parsing process

					// Sending the message
					$context->sendMessage('⚠️ *Не запущен процесс генерации парсинга*')
						->then(function (message $message) use ($context) {
							// Sended the message

							// Sending the menu with subjects
							commands::menu($context);
						});
				}
			});
	}
}
